/*
 * data.c - Values, variables, and operators
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */


#include "dwarfx.h"

#include "util.h"
#include "data.h"


extern void yyerror(const char *s);

const struct dwx_type *void_type;
const struct dwx_type *int8_type;
const struct dwx_type *int16_type;
const struct dwx_type *int32_type;
const struct dwx_type *int64_type;
const struct dwx_type *uint8_type;
const struct dwx_type *uint16_type;
const struct dwx_type *uint32_type;
const struct dwx_type *uint64_type;

/*
 * Memory objects that identify types which are private to the script
 * processor, and don't exist in this form in C.
 */

struct dwx_type type_type;
struct dwx_type string_type;


struct value int_value(uint32_t num)
{
    struct value value;

    value.type = uint32_type;
    /* @@@ memory leak. should reference count. */
    value.value = alloc_t(uint32_t);
    *(uint32_t *) value.value = num;
    return value;
}


uint32_t value_int(struct value value)
{
    if (value.type->kind != dwx_uint && value.type->kind != dwx_sint)
        yyerror("invalid type (expected integer)");
    return *(uint32_t *) value.value;
}


uint32_t int_eval(const struct op *op)
{
    return value_int(op->fn(op));
}


struct value string_value(const char *str)
{
    struct value value;

    value.type = &string_type;
    value.value = str;
    return value;
}


const char *value_string(struct value value)
{
    if (value.type != &string_type) yyerror("invalid type (expected string)");
    return value.value;
}


const char *string_eval(const struct op *op)
{
    return value_string(op->fn(op));
}


struct value type_value(const struct dwx_type *type)
{
    struct value value;

    value.type = &type_type;
    value.value = type;
    return value;
}


const struct dwx_type *value_type(struct value value)
{
    if (value.type != &type_type) yyerror("invalid type (expected type)");
    return value.value;
}


const struct dwx_type *type_eval(const struct op *op)
{
    return value_type(op->fn(op));
}


struct value evaluate(const struct op *op)
{
     return op->fn(op);
}


void init_types(void)
{
    int8_type = dwx_basic_type(dwx_sint,1);
    int16_type = dwx_basic_type(dwx_sint,2);
    int32_type = dwx_basic_type(dwx_sint,4);
    int64_type = dwx_basic_type(dwx_sint,8);
    uint8_type = dwx_basic_type(dwx_uint,1);
    uint16_type = dwx_basic_type(dwx_uint,2);
    uint32_type = dwx_basic_type(dwx_uint,4);
    uint64_type = dwx_basic_type(dwx_uint,8);
}
